@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 """
 =====================
 Krackhardt Centrality
@@ -6,16 +5,6 @@ Krackhardt Centrality
 
 Centrality measures of Krackhardt social network.
 """
-# Author: Aric Hagberg (hagberg@lanl.gov)
-# Date: 2005-05-12 14:33:11 -0600 (Thu, 12 May 2005)
-# Revision: 998
-
-#    Copyright (C) 2004-2019 by
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    All rights reserved.
-#    BSD license.
 
 import matplotlib.pyplot as plt
 import networkx as nx
@@ -25,17 +14,18 @@ G = nx.krackhardt_kite_graph()
 print("Betweenness")
 b = nx.betweenness_centrality(G)
 for v in G.nodes():
-    print("%0.2d %5.3f" % (v, b[v]))
+    print(f"{v:2} {b[v]:.3f}")
 
 print("Degree centrality")
 d = nx.degree_centrality(G)
 for v in G.nodes():
-    print("%0.2d %5.3f" % (v, d[v]))
+    print(f"{v:2} {d[v]:.3f}")
 
 print("Closeness centrality")
 c = nx.closeness_centrality(G)
 for v in G.nodes():
-    print("%0.2d %5.3f" % (v, c[v]))
+    print(f"{v:2} {c[v]:.3f}")
 
-nx.draw(G)
+pos = nx.spring_layout(G, seed=367)  # Seed layout for reproducibility
+nx.draw(G, pos)
 plt.show()
