@@ -1,30 +1,24 @@
-# __init__.py - package containing heuristics for optimization problems
-#
-# Copyright 2016-2019 NetworkX developers.
-#
-# This file is part of NetworkX.
-#
-# NetworkX is distributed under a BSD license; see LICENSE.txt for more
-# information.
-"""Approximations of graph properties and Heuristic functions for optimization
-problems.
+"""Approximations of graph properties and Heuristic methods for optimization.
 
-    .. warning:: The approximation submodule is not imported in the top-level
-        ``networkx``.
+The functions in this class are not imported into the top-level ``networkx``
+namespace so the easiest way to use them is with::
 
-    These functions can be imported with
-    ``from networkx.algorithms import approximation``.
+    >>> from networkx.algorithms import approximation
+
+Another option is to import the specific function with
+``from networkx.algorithms.approximation import function_name``.
 
 """
-
 from networkx.algorithms.approximation.clustering_coefficient import *
 from networkx.algorithms.approximation.clique import *
 from networkx.algorithms.approximation.connectivity import *
+from networkx.algorithms.approximation.distance_measures import *
 from networkx.algorithms.approximation.dominating_set import *
 from networkx.algorithms.approximation.kcomponents import *
-from networkx.algorithms.approximation.independent_set import *
 from networkx.algorithms.approximation.matching import *
 from networkx.algorithms.approximation.ramsey import *
 from networkx.algorithms.approximation.steinertree import *
+from networkx.algorithms.approximation.traveling_salesman import *
+from networkx.algorithms.approximation.treewidth import *
 from networkx.algorithms.approximation.vertex_cover import *
-from networkx.algorithms.approximation.treewidth import *
\ No newline at end of file
+from networkx.algorithms.approximation.maxcut import *
