@@ -1,14 +1,11 @@
 """Functions for computing and measuring community structure.
 
-The functions in this class are not imported into the top-level
-:mod:`networkx` namespace. You can access these functions by importing
-the :mod:`networkx.algorithms.community` module, then accessing the
+The ``community`` subpackage can be accessed by using :mod:`networkx.community`, then accessing the
 functions as attributes of ``community``. For example::
 
     >>> import networkx as nx
-    >>> from networkx.algorithms import community
     >>> G = nx.barbell_graph(5, 1)
-    >>> communities_generator = community.girvan_newman(G)
+    >>> communities_generator = nx.community.girvan_newman(G)
     >>> top_level_communities = next(communities_generator)
     >>> next_level_communities = next(communities_generator)
     >>> sorted(map(sorted, next_level_communities))
@@ -20,6 +17,8 @@ from networkx.algorithms.community.centrality import *
 from networkx.algorithms.community.kclique import *
 from networkx.algorithms.community.kernighan_lin import *
 from networkx.algorithms.community.label_propagation import *
+from networkx.algorithms.community.lukes import *
 from networkx.algorithms.community.modularity_max import *
 from networkx.algorithms.community.quality import *
 from networkx.algorithms.community.community_utils import *
+from networkx.algorithms.community.louvain import *
