@@ -1,23 +1,19 @@
-#    Copyright (C) 2016-2019 by
-#    Minas Gjoka
-#    Balint Tillman
-#    BSD license.
-#
-# Authors:  Minas Gjoka (minas.gjoka@gmail.com)
-#           Balint Tillman (tillman.balint@gmail.com)
 """Generate graphs with a given joint degree and directed joint degree"""
 
 import networkx as nx
 from networkx.utils import py_random_state
 
-__all__ = ['is_valid_joint_degree',
-           'is_valid_directed_joint_degree',
-           'joint_degree_graph',
-           'directed_joint_degree_graph']
+__all__ = [
+    "is_valid_joint_degree",
+    "is_valid_directed_joint_degree",
+    "joint_degree_graph",
+    "directed_joint_degree_graph",
+]
 
 
+@nx._dispatch(graphs=None)
 def is_valid_joint_degree(joint_degrees):
-    """ Checks whether the given joint degree dictionary is realizable.
+    """Checks whether the given joint degree dictionary is realizable.
 
     A *joint degree dictionary* is a dictionary of dictionaries, in
     which entry ``joint_degrees[k][l]`` is an integer representing the
@@ -69,12 +65,10 @@ def is_valid_joint_degree(joint_degrees):
             if not float(joint_degrees[k][l]).is_integer():
                 return False
 
-            if (k != l) and (joint_degrees[k][l] >
-                             degree_count[k] * degree_count[l]):
+            if (k != l) and (joint_degrees[k][l] > degree_count[k] * degree_count[l]):
                 return False
             elif k == l:
-                if (joint_degrees[k][k] > degree_count[k] *
-                        (degree_count[k] - 1)):
+                if joint_degrees[k][k] > degree_count[k] * (degree_count[k] - 1):
                     return False
                 if joint_degrees[k][k] % 2 != 0:
                     return False
@@ -85,7 +79,7 @@ def is_valid_joint_degree(joint_degrees):
 
 
 def _neighbor_switch(G, w, unsat, h_node_residual, avoid_node_id=None):
-    """ Releases one free stub for ``w``, while preserving joint degree in G.
+    """Releases one free stub for ``w``, while preserving joint degree in G.
 
     Parameters
     ----------
@@ -115,7 +109,7 @@ def _neighbor_switch(G, w, unsat, h_node_residual, avoid_node_id=None):
     """
 
     if (avoid_node_id is None) or (h_node_residual[avoid_node_id] > 1):
-        # select unsatured node w_prime that has the same degree as w
+        # select unsaturated node w_prime that has the same degree as w
         w_prime = next(iter(unsat))
     else:
         # assume that the node pair (v,w) has been selected for connection. if
@@ -148,8 +142,9 @@ def _neighbor_switch(G, w, unsat, h_node_residual, avoid_node_id=None):
 
 
 @py_random_state(1)
+@nx._dispatch(graphs=None)
 def joint_degree_graph(joint_degrees, seed=None):
-    """ Generates a random simple graph with the given joint degree dictionary.
+    """Generates a random simple graph with the given joint degree dictionary.
 
     Parameters
     ----------
@@ -195,22 +190,22 @@ def joint_degree_graph(joint_degrees, seed=None):
 
     Examples
     --------
-    >>> import networkx as nx
-    >>> joint_degrees = {1: {4: 1},
-    ...                      2: {2: 2, 3: 2, 4: 2},
-    ...                      3: {2: 2, 4: 1},
-    ...                      4: {1: 1, 2: 2, 3: 1}}
-    >>> G=nx.joint_degree_graph(joint_degrees)
+    >>> joint_degrees = {
+    ...     1: {4: 1},
+    ...     2: {2: 2, 3: 2, 4: 2},
+    ...     3: {2: 2, 4: 1},
+    ...     4: {1: 1, 2: 2, 3: 1},
+    ... }
+    >>> G = nx.joint_degree_graph(joint_degrees)
     >>>
     """
 
     if not is_valid_joint_degree(joint_degrees):
-        msg = 'Input joint degree dict not realizable as a simple graph'
+        msg = "Input joint degree dict not realizable as a simple graph"
         raise nx.NetworkXError(msg)
 
     # compute degree count from joint_degrees
-    degree_count = {k: sum(l.values()) // k for k, l in joint_degrees.items()
-                    if k > 0}
+    degree_count = {k: sum(l.values()) // k for k, l in joint_degrees.items() if k > 0}
 
     # start with empty N-node graph
     N = sum(degree_count.values())
@@ -234,13 +229,11 @@ def joint_degree_graph(joint_degrees, seed=None):
     # for each pair
     for k in joint_degrees:
         for l in joint_degrees[k]:
-
             # n_edges_add is the number of edges to add for the
             # degree pair (k,l)
             n_edges_add = joint_degrees[k][l]
 
             if (n_edges_add > 0) and (k >= l):
-
                 # number of nodes with degree k and l
                 k_size = degree_count[k]
                 l_size = degree_count[l]
@@ -251,23 +244,21 @@ def joint_degree_graph(joint_degrees, seed=None):
 
                 # k_unsat and l_unsat consist of nodes of degree k and l that
                 # are unsaturated (nodes that have at least 1 available stub)
-                k_unsat = set(v for v in k_nodes if h_node_residual[v] > 0)
+                k_unsat = {v for v in k_nodes if h_node_residual[v] > 0}
 
                 if k != l:
-                    l_unsat = set(w for w in l_nodes if h_node_residual[w] > 0)
+                    l_unsat = {w for w in l_nodes if h_node_residual[w] > 0}
                 else:
                     l_unsat = k_unsat
                     n_edges_add = joint_degrees[k][l] // 2
 
                 while n_edges_add > 0:
-
                     # randomly pick nodes v and w that have degrees k and l
                     v = k_nodes[seed.randrange(k_size)]
                     w = l_nodes[seed.randrange(l_size)]
 
                     # if nodes v and w are disconnected then attempt to connect
                     if not G.has_edge(v, w) and (v != w):
-
                         # if node v has no free stubs then do neighbor switch
                         if h_node_residual[v] == 0:
                             _neighbor_switch(G, v, k_unsat, h_node_residual)
@@ -275,12 +266,11 @@ def joint_degree_graph(joint_degrees, seed=None):
                         # if node w has no free stubs then do neighbor switch
                         if h_node_residual[w] == 0:
                             if k != l:
-                                _neighbor_switch(G, w, l_unsat,
-                                                 h_node_residual)
+                                _neighbor_switch(G, w, l_unsat, h_node_residual)
                             else:
-                                _neighbor_switch(G, w, l_unsat,
-                                                 h_node_residual,
-                                                 avoid_node_id=v)
+                                _neighbor_switch(
+                                    G, w, l_unsat, h_node_residual, avoid_node_id=v
+                                )
 
                         # add edge (v, w) and update data structures
                         G.add_edge(v, w)
@@ -295,8 +285,9 @@ def joint_degree_graph(joint_degrees, seed=None):
     return G
 
 
+@nx._dispatch(graphs=None)
 def is_valid_directed_joint_degree(in_degrees, out_degrees, nkk):
-    """ Checks whether the given directed joint degree input is realizable
+    """Checks whether the given directed joint degree input is realizable
 
     Parameters
     ----------
@@ -306,7 +297,7 @@ def is_valid_directed_joint_degree(in_degrees, out_degrees, nkk):
         out degree sequence contains the out degrees of nodes.
     nkk  :  dictionary of dictionary of integers
         directed joint degree dictionary. for nodes of out degree k (first
-        level of dict) and nodes of in degree l (seconnd level of dict)
+        level of dict) and nodes of in degree l (second level of dict)
         describes the number of edges.
 
     Returns
@@ -337,7 +328,7 @@ def is_valid_directed_joint_degree(in_degrees, out_degrees, nkk):
     if len(in_degrees) != len(out_degrees):
         return False
 
-    for idx in range(0, len(in_degrees)):
+    for idx in range(len(in_degrees)):
         i = in_degrees[idx]
         o = out_degrees[idx]
         V[(i, 0)] = V.get((i, 0), 0) + 1
@@ -359,18 +350,13 @@ def is_valid_directed_joint_degree(in_degrees, out_degrees, nkk):
                 if val + forbidden.get((k, l), 0) > V[(k, 1)] * V[(l, 0)]:
                     return False
 
-    for s in S:
-        if not float(S[s]) / s[0] == V[s]:  # condition 2
-            return False
-
-    # if all conditions abive have been satisfied then the input nkk is
-    # realizable as a simple graph.
-    return True
+    return all(S[s] / s[0] == V[s] for s in S)
 
 
-def _directed_neighbor_switch(G, w, unsat, h_node_residual_out, chords,
-                              h_partition_in, partition):
-    """ Releases one free stub for node w, while preserving joint degree in G.
+def _directed_neighbor_switch(
+    G, w, unsat, h_node_residual_out, chords, h_partition_in, partition
+):
+    """Releases one free stub for node w, while preserving joint degree in G.
 
     Parameters
     ----------
@@ -430,9 +416,10 @@ def _directed_neighbor_switch(G, w, unsat, h_node_residual_out, chords,
     return w_prime
 
 
-def _directed_neighbor_switch_rev(G, w, unsat, h_node_residual_in, chords,
-                                  h_partition_out, partition):
-    """ The reverse of directed_neighbor_switch.
+def _directed_neighbor_switch_rev(
+    G, w, unsat, h_node_residual_in, chords, h_partition_out, partition
+):
+    """The reverse of directed_neighbor_switch.
 
     Parameters
     ----------
@@ -482,8 +469,9 @@ def _directed_neighbor_switch_rev(G, w, unsat, h_node_residual_in, chords,
 
 
 @py_random_state(3)
+@nx._dispatch(graphs=None)
 def directed_joint_degree_graph(in_degrees, out_degrees, nkk, seed=None):
-    """ Generates a random simple directed graph with the joint degree.
+    """Generates a random simple directed graph with the joint degree.
 
     Parameters
     ----------
@@ -540,15 +528,14 @@ def directed_joint_degree_graph(in_degrees, out_degrees, nkk, seed=None):
 
     Examples
     --------
-    >>> import networkx as nx
     >>> in_degrees = [0, 1, 1, 2]
     >>> out_degrees = [1, 1, 1, 1]
-    >>> nkk = {1:{1:2,2:2}}
-    >>> G=nx.directed_joint_degree_graph(in_degrees, out_degrees, nkk)
+    >>> nkk = {1: {1: 2, 2: 2}}
+    >>> G = nx.directed_joint_degree_graph(in_degrees, out_degrees, nkk)
     >>>
     """
     if not is_valid_directed_joint_degree(in_degrees, out_degrees, nkk):
-        msg = 'Input is not realizable as a simple graph'
+        msg = "Input is not realizable as a simple graph"
         raise nx.NetworkXError(msg)
 
     # start with an empty directed graph.
@@ -582,8 +569,7 @@ def directed_joint_degree_graph(in_degrees, out_degrees, nkk, seed=None):
 
     for idx, o in enumerate(out_degrees):
         o = out_degrees[idx]
-        non_chords[(o, in_degrees[idx])] = non_chords.get((o, in_degrees[idx]),
-                                                          0) + 1
+        non_chords[(o, in_degrees[idx])] = non_chords.get((o, in_degrees[idx]), 0) + 1
         idx = int(idx)
         if o > 0:
             h_degree_nodelist_out.setdefault(o, [])
@@ -608,14 +594,15 @@ def directed_joint_degree_graph(in_degrees, out_degrees, nkk, seed=None):
         for l in nkk[k]:
             n_edges_add = nkk[k][l]
 
-            if (n_edges_add > 0):
+            if n_edges_add > 0:
                 # chords contains a random set of potential edges.
                 chords = set()
 
                 k_len = nk_out[k]
                 l_len = nk_in[l]
-                chords_sample = seed.sample(range(k_len * l_len), n_edges_add
-                                            + non_chords.get((k, l), 0))
+                chords_sample = seed.sample(
+                    range(k_len * l_len), n_edges_add + non_chords.get((k, l), 0)
+                )
 
                 num = 0
                 while len(chords) < n_edges_add:
@@ -637,19 +624,29 @@ def directed_joint_degree_graph(in_degrees, out_degrees, nkk, seed=None):
 
                     # if node v has no free stubs then do neighbor switch.
                     if h_node_residual_out[v] == 0:
-                        _v = _directed_neighbor_switch(G, v, k_unsat,
-                                                       h_node_residual_out,
-                                                       chords, h_partition_in,
-                                                       l)
+                        _v = _directed_neighbor_switch(
+                            G,
+                            v,
+                            k_unsat,
+                            h_node_residual_out,
+                            chords,
+                            h_partition_in,
+                            l,
+                        )
                         if _v is not None:
                             v = _v
 
                     # if node w has no free stubs then do neighbor switch.
                     if h_node_residual_in[w] == 0:
-                        _w = _directed_neighbor_switch_rev(G, w, l_unsat,
-                                                           h_node_residual_in,
-                                                           chords,
-                                                           h_partition_out, k)
+                        _w = _directed_neighbor_switch_rev(
+                            G,
+                            w,
+                            l_unsat,
+                            h_node_residual_in,
+                            chords,
+                            h_partition_out,
+                            k,
+                        )
                         if _w is not None:
                             w = _w
 
